@@ -79,6 +79,7 @@ send_response_in_thread_func (GTask        *task,
   const char **uris;
   GVariant *choices;
   GVariant *current_filter;
+  GVariant *writable;
 
   g_variant_builder_init (&results, G_VARIANT_TYPE_VARDICT);
   g_variant_builder_init (&ruris, G_VARIANT_TYPE_STRING_ARRAY);
@@ -95,7 +96,8 @@ send_response_in_thread_func (GTask        *task,
   if (response != 0)
     goto out;
 
-  if (!g_variant_lookup_value (options, "writable", G_VARIANT_TYPE("b")))
+  writable = g_variant_lookup_value (options, "writable", G_VARIANT_TYPE("b"));
+  if (writable && !g_variant_get_boolean (writable))
     flags &= ~DOCUMENT_FLAG_WRITABLE;
 
   choices = g_variant_lookup_value (options, "choices", G_VARIANT_TYPE ("a(ss)"));
@@ -142,6 +144,53 @@ out:
     }
 }
 
+/* Calling Lookup on a nonexisting path does not work, so we
+ * pull the doc id out of the path manually.
+ */
+static gboolean
+looks_like_document_portal_path (const char *path,
+                                 char **guessed_docid)
+{
+  const char *prefix = "/run/user/";
+  char *docid;
+  char *p, *q;
+
+  if (!g_str_has_prefix (path, prefix))
+    return FALSE;
+
+  p = strstr (path, "/doc/");
+  if (!p)
+    return FALSE;
+
+  p += strlen ("/doc/");
+  q = strchr (p, '/');
+  if (q)
+    docid = g_strndup (p, q - p);
+  else
+    docid = g_strdup (p);
+
+  if (docid[0] == '\0')
+    {
+      g_free (docid);
+      return FALSE;
+    }
+
+  *guessed_docid = docid;
+  return TRUE;
+}
+
+static char *
+get_host_folder_for_doc_id (const char *doc_id)
+{
+  g_autofree char *real_path = get_real_path_for_doc_id (doc_id);
+  g_autofree char *host_folder = NULL;
+
+  if (real_path != NULL)
+    host_folder = g_path_get_dirname (real_path);
+
+  return g_steal_pointer (&host_folder);
+}
+
 static void
 open_file_done (GObject *source,
                 GAsyncResult *result,
@@ -448,6 +497,7 @@ validate_choices (const char *key,
   return TRUE;
 }
 
+/* Note: current_folder is intentionally left out */
 static XdpOptionKey open_file_options[] = {
   { "accept_label", G_VARIANT_TYPE_STRING, NULL },
   { "modal", G_VARIANT_TYPE_BOOLEAN, NULL },
@@ -455,7 +505,7 @@ static XdpOptionKey open_file_options[] = {
   { "directory", G_VARIANT_TYPE_BOOLEAN, NULL },
   { "filters", (const GVariantType *)"a(sa(us))", validate_filters },
   { "current_filter", (const GVariantType *)"(sa(us))", validate_current_filter },
-  { "choices", (const GVariantType *)"a(ssa(ss)s)", validate_choices }
+  { "choices", (const GVariantType *)"a(ssa(ss)s)", validate_choices },
 };
 
 static gboolean
@@ -485,6 +535,30 @@ handle_open_file (XdpDbusFileChooser *object,
       return G_DBUS_METHOD_INVOCATION_HANDLED;
     }
 
+  {
+    g_autoptr(GVariant) value =
+      g_variant_lookup_value (arg_options, "current_folder", G_VARIANT_TYPE_BYTESTRING);
+
+    if (value)
+      {
+        const char *path_from_app = g_variant_get_bytestring (value);
+        g_autofree char *host_path = g_strdup (path_from_app);
+        g_autofree char *doc_id_from_app = NULL;
+        if (looks_like_document_portal_path (host_path, &doc_id_from_app))
+          {
+            char *real_path = get_host_folder_for_doc_id (doc_id_from_app);
+            if (real_path)
+              {
+                g_free (host_path);
+                host_path = real_path;
+              }
+            g_debug ("OpenFile: translating current_folder value '%s' to host path '%s'", path_from_app, host_path);
+          }
+        g_variant_builder_add (&options, "{sv}", "current_folder",
+                              g_variant_new_bytestring (host_path));
+      }
+  }
+
   impl_request =
     xdp_dbus_impl_request_proxy_new_sync (g_dbus_proxy_get_connection (G_DBUS_PROXY (impl)),
                                           G_DBUS_PROXY_FLAGS_NONE,
@@ -521,15 +595,16 @@ handle_open_file (XdpDbusFileChooser *object,
   return G_DBUS_METHOD_INVOCATION_HANDLED;
 }
 
+/* Note that current_file and current_folder are intentionally left out here.
+ * It is handled separately below
+ */
 static XdpOptionKey save_file_options[] = {
   { "accept_label", G_VARIANT_TYPE_STRING, NULL },
   { "modal", G_VARIANT_TYPE_BOOLEAN, NULL },
   { "filters", (const GVariantType *)"a(sa(us))", validate_filters },
   { "current_filter", (const GVariantType *)"(sa(us))", validate_current_filter },
   { "current_name", G_VARIANT_TYPE_STRING, NULL },
-  { "current_folder", G_VARIANT_TYPE_BYTESTRING, NULL },
-  { "current_file", G_VARIANT_TYPE_BYTESTRING, NULL },
-  { "choices", (const GVariantType *)"a(ssa(ss)s)", validate_choices  }
+  { "choices", (const GVariantType *)"a(ssa(ss)s)", validate_choices }
 };
 
 static void
@@ -598,6 +673,58 @@ handle_save_file (XdpDbusFileChooser *object,
       return G_DBUS_METHOD_INVOCATION_HANDLED;
     }
 
+  {
+    g_autoptr(GVariant) value = g_variant_lookup_value (arg_options,
+                                                        "current_file",
+                                                        G_VARIANT_TYPE_BYTESTRING);
+
+    if (value)
+      {
+        const char *path = g_variant_get_bytestring (value);
+        g_autofree char *host_path = get_real_path_for_doc_path (path, request->app_info);
+        g_autofree char *doc_id = NULL;
+
+        if (strcmp (path, host_path) == 0 &&
+            looks_like_document_portal_path (path, &doc_id))
+          {
+            char *real_path = get_real_path_for_doc_id (doc_id);
+
+            if (real_path)
+              {
+                g_free (host_path);
+                host_path = real_path;
+              }
+          }
+
+        g_debug ("SaveFile: translating current_file value '%s' to host path '%s'", path, host_path);
+
+        g_variant_builder_add (&options, "{sv}", "current_file", g_variant_new_bytestring (host_path));
+      }
+  }
+  {
+      g_autoptr(GVariant) value =
+        g_variant_lookup_value (arg_options, "current_folder", G_VARIANT_TYPE_BYTESTRING);
+
+      if (value)
+        {
+          const char *path_from_app = g_variant_get_bytestring (value);
+          g_autofree char *host_path = g_strdup (path_from_app);
+          g_autofree char *doc_id_from_app = NULL;
+          if (looks_like_document_portal_path (host_path, &doc_id_from_app))
+            {
+              char *real_path = get_host_folder_for_doc_id (doc_id_from_app);
+              if (real_path)
+                {
+                  g_free (host_path);
+                  host_path = real_path;
+                }
+              g_debug ("SaveFile: translating current_folder value '%s' to host path '%s'", path_from_app, host_path);
+            }
+          g_variant_builder_add (&options, "{sv}", "current_folder",
+                                g_variant_new_bytestring (host_path));
+        }
+  }
+
   impl_request =
     xdp_dbus_impl_request_proxy_new_sync (g_dbus_proxy_get_connection (G_DBUS_PROXY (impl)),
                                           G_DBUS_PROXY_FLAGS_NONE,
@@ -625,7 +752,7 @@ handle_save_file (XdpDbusFileChooser *object,
                                              save_file_done,
                                              g_object_ref (request));
 
-  xdp_dbus_file_chooser_complete_open_file (object, invocation, request->id);
+  xdp_dbus_file_chooser_complete_save_file (object, invocation, request->id);
 
   return G_DBUS_METHOD_INVOCATION_HANDLED;
 }
@@ -730,7 +857,7 @@ handle_save_files (XdpDbusFileChooser *object,
                                               save_files_done,
                                               g_object_ref (request));
 
-  xdp_dbus_file_chooser_complete_open_file (object, invocation, request->id);
+  xdp_dbus_file_chooser_complete_save_files (object, invocation, request->id);
 
   return G_DBUS_METHOD_INVOCATION_HANDLED;
 }
