@@ -22,6 +22,15 @@
 
 #include <string.h>
 
+enum
+{
+  INTERNAL_CLOSED,
+
+  N_SIGNALS
+};
+
+static guint signals[N_SIGNALS];
+
 enum
 {
   PROP_0,
@@ -174,6 +183,8 @@ session_close (Session *session,
 
   SESSION_GET_CLASS (session)->close (session);
 
+  g_signal_emit (session, signals[INTERNAL_CLOSED], 0);
+
   if (notify_closed)
     {
       GVariantBuilder details_builder;
@@ -528,4 +539,11 @@ session_class_init (SessionClass *klass)
                          G_PARAM_STATIC_STRINGS);
 
   g_object_class_install_properties (gobject_class, PROP_LAST, obj_props);
+
+  signals[INTERNAL_CLOSED] = g_signal_new ("internal-closed",
+                                           G_TYPE_FROM_CLASS (klass),
+                                           G_SIGNAL_RUN_LAST,
+                                           0,
+                                           NULL, NULL, NULL,
+                                           G_TYPE_NONE, 0);
 }
