@@ -34,6 +34,7 @@
 #include "background.h"
 #include "call.h"
 #include "camera.h"
+#include "clipboard.h"
 #include "device.h"
 #include "documents.h"
 #include "dynamic-launcher.h"
@@ -42,6 +43,7 @@
 #include "gamemode.h"
 #include "global-shortcuts.h"
 #include "inhibit.h"
+#include "input-capture.h"
 #include "location.h"
 #include "memory-monitor.h"
 #include "network-monitor.h"
@@ -120,11 +122,15 @@ method_needs_request (GDBusMethodInvocation *invocation)
     }
   else if (strcmp (interface, "org.freedesktop.portal.RemoteDesktop") == 0)
     {
-      if (strstr (method, "Notify") == method)
+      if (strstr (method, "Notify") == method || strcmp (method, "ConnectToEIS") == 0)
         return FALSE;
       else
         return TRUE;
     }
+  else if (strcmp (interface, "org.freedesktop.portal.Clipboard") == 0)
+    {
+      return FALSE;
+    }
   else if (strcmp (interface, "org.freedesktop.portal.Camera") == 0)
     {
       if (strcmp (method, "OpenPipeWireRemote") == 0)
@@ -146,6 +152,16 @@ method_needs_request (GDBusMethodInvocation *invocation)
       else
         return TRUE;
     }
+  else if (strcmp (interface, "org.freedesktop.portal.InputCapture") == 0)
+    {
+      if (strcmp (method, "ConnectToEIS") == 0 ||
+          strcmp (method, "Enable") == 0 ||
+          strcmp (method, "Disable") == 0 ||
+          strcmp (method, "Release") == 0)
+        return FALSE;
+      else
+        return TRUE;
+    }
   else
     {
       return TRUE;
@@ -213,9 +229,6 @@ peer_died_cb (const char *name)
 {
   close_requests_for_sender (name);
   close_sessions_for_sender (name);
-#ifdef HAVE_PIPEWIRE
-  screen_cast_remove_transient_permissions_for_sender (name);
-#endif
 }
 
 static void
@@ -224,7 +237,8 @@ on_bus_acquired (GDBusConnection *connection,
                  gpointer         user_data)
 {
   PortalImplementation *implementation;
-  PortalImplementation *implementation2;
+  PortalImplementation *lockdown_impl;
+  PortalImplementation *access_impl;
   g_autoptr(GError) error = NULL;
   XdpDbusImplLockdown *lockdown;
   GQuark portal_errors G_GNUC_UNUSED;
@@ -237,11 +251,11 @@ on_bus_acquired (GDBusConnection *connection,
   init_document_proxy (connection);
   init_permission_store (connection);
 
-  implementation = find_portal_implementation ("org.freedesktop.impl.portal.Lockdown");
-  if (implementation != NULL)
+  lockdown_impl = find_portal_implementation ("org.freedesktop.impl.portal.Lockdown");
+  if (lockdown_impl != NULL)
     lockdown = xdp_dbus_impl_lockdown_proxy_new_sync (connection,
                                                       G_DBUS_PROXY_FLAGS_NONE,
-                                                      implementation->dbus_name,
+                                                      lockdown_impl->dbus_name,
                                                       DESKTOP_PORTAL_OBJECT_PATH,
                                                       NULL, &error);
   else
@@ -284,42 +298,47 @@ on_bus_acquired (GDBusConnection *connection,
     export_portal_implementation (connection,
                                   inhibit_create (connection, implementation->dbus_name));
 
-  implementation = find_portal_implementation ("org.freedesktop.impl.portal.Access");
-  implementation2 = find_portal_implementation ("org.freedesktop.impl.portal.Screenshot");
-  if (implementation != NULL && implementation2 != NULL)
-    export_portal_implementation (connection,
-                                  screenshot_create (connection,
-                                                     implementation->dbus_name,
-                                                     implementation2->dbus_name));
-
-  implementation2 = find_portal_implementation ("org.freedesktop.impl.portal.Background");
-  if (implementation != NULL)
+  access_impl = find_portal_implementation ("org.freedesktop.impl.portal.Access");
+  if (access_impl != NULL)
     {
+      PortalImplementation *tmp;
+
       export_portal_implementation (connection,
-                                    device_create (connection, implementation->dbus_name, lockdown));
+                                    device_create (connection,
+                                                   access_impl->dbus_name,
+                                                   lockdown));
 #ifdef HAVE_GEOCLUE
       export_portal_implementation (connection,
-                                    location_create (connection, implementation->dbus_name, lockdown));
+                                    location_create (connection,
+                                                     access_impl->dbus_name,
+                                                     lockdown));
 #endif
 
-#ifdef HAVE_PIPEWIRE
-      export_portal_implementation (connection, camera_create (connection, lockdown));
-#endif
+      export_portal_implementation (connection,
+                                    camera_create (connection, lockdown));
+
+      tmp = find_portal_implementation ("org.freedesktop.impl.portal.Screenshot");
+      if (tmp != NULL)
+        export_portal_implementation (connection,
+                                      screenshot_create (connection,
+                                                         access_impl->dbus_name,
+                                                         tmp->dbus_name));
+
+      tmp = find_portal_implementation ("org.freedesktop.impl.portal.Background");
+      if (tmp != NULL)
+        export_portal_implementation (connection,
+                                      background_create (connection,
+                                                         access_impl->dbus_name,
+                                                         tmp->dbus_name));
+
+      tmp = find_portal_implementation ("org.freedesktop.impl.portal.Wallpaper");
+      if (tmp != NULL)
+        export_portal_implementation (connection,
+                                      wallpaper_create (connection,
+                                                        access_impl->dbus_name,
+                                                        tmp->dbus_name));
     }
 
-  if (implementation != NULL && implementation2 != NULL)
-    export_portal_implementation (connection,
-                                  background_create (connection,
-                                                     implementation->dbus_name,
-                                                     implementation2->dbus_name));
-
-  implementation2 = find_portal_implementation ("org.freedesktop.impl.portal.Wallpaper");
-  if (implementation != NULL && implementation2 != NULL)
-    export_portal_implementation (connection,
-                                  wallpaper_create (connection,
-                                                    implementation->dbus_name,
-                                                    implementation2->dbus_name));
-
   implementation = find_portal_implementation ("org.freedesktop.impl.portal.Account");
   if (implementation != NULL)
     export_portal_implementation (connection,
@@ -340,14 +359,11 @@ on_bus_acquired (GDBusConnection *connection,
     export_portal_implementation (connection,
                                   global_shortcuts_create (connection, implementation->dbus_name));
 
-#ifdef HAVE_GLIB_2_66
   implementation = find_portal_implementation ("org.freedesktop.impl.portal.DynamicLauncher");
   if (implementation != NULL)
     export_portal_implementation (connection,
                                   dynamic_launcher_create (connection, implementation->dbus_name));
-#endif
 
-#ifdef HAVE_PIPEWIRE
   implementation = find_portal_implementation ("org.freedesktop.impl.portal.ScreenCast");
   if (implementation != NULL)
     export_portal_implementation (connection,
@@ -357,7 +373,16 @@ on_bus_acquired (GDBusConnection *connection,
   if (implementation != NULL)
     export_portal_implementation (connection,
                                   remote_desktop_create (connection, implementation->dbus_name));
-#endif
+
+  implementation = find_portal_implementation ("org.freedesktop.impl.portal.Clipboard");
+  if (implementation != NULL)
+    export_portal_implementation (
+        connection, clipboard_create (connection, implementation->dbus_name));
+
+  implementation = find_portal_implementation ("org.freedesktop.impl.portal.InputCapture");
+  if (implementation != NULL)
+    export_portal_implementation (connection,
+                                  input_capture_create (connection, implementation->dbus_name));
 }
 
 static void
@@ -434,6 +459,7 @@ main (int argc, char *argv[])
 
   g_set_prgname (argv[0]);
 
+  load_portal_configuration (opt_verbose);
   load_installed_portals (opt_verbose);
 
   loop = g_main_loop_new (NULL, FALSE);
