@@ -67,8 +67,8 @@ xdp_mkstempat (int    dir_fd,
   static const char letters[] =
     "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
   static const int NLETTERS = sizeof (letters) - 1;
-  glong value;
-  GTimeVal tv;
+  gint64 value;
+  gint64 current_time;
   static int counter = 0;
 
   g_return_val_if_fail (tmpl != NULL, -1);
@@ -83,12 +83,12 @@ xdp_mkstempat (int    dir_fd,
     }
 
   /* Get some more or less random data.  */
-  g_get_current_time (&tv);
-  value = (tv.tv_usec ^ tv.tv_sec) + counter++;
+  current_time = g_get_real_time ();
+  value = ((current_time % G_USEC_PER_SEC) ^ (current_time / G_USEC_PER_SEC)) + counter++;
 
   for (count = 0; count < 100; value += 7777, ++count)
     {
-      glong v = value;
+      gint64 v = value;
 
       /* Fill in the random bits.  */
       XXXXXX[0] = letters[v % NLETTERS];
@@ -153,7 +153,8 @@ xdp_app_info_new (XdpAppInfoKind kind)
 char *
 _xdp_parse_app_id_from_unit_name (const char *unit)
 {
-  g_autoptr(GRegex) regex = NULL;
+  g_autoptr(GRegex) regex1 = NULL;
+  g_autoptr(GRegex) regex2 = NULL;
   g_autoptr(GMatchInfo) match = NULL;
   g_autoptr(GError) error = NULL;
   g_autofree char *app_id = NULL;
@@ -163,16 +164,24 @@ _xdp_parse_app_id_from_unit_name (const char *unit)
   /*
    * From https://systemd.io/DESKTOP_ENVIRONMENTS/ the format is one of:
    * app[-<launcher>]-<ApplicationID>-<RANDOM>.scope
-   * app[-<launcher>]-<ApplicationID>-autostart.service -> no longer true since systemd v248
-   * app[-<launcher>]-<ApplicationID>[@<RANDOM>].service
    * app[-<launcher>]-<ApplicationID>-<RANDOM>.slice
    */
-  regex = g_regex_new ("^app-(?:[[:alnum:]]+\\-)?(.+?\\..+?\\..+?)"
-                       "(?:[@\\-][[:alnum:]]*)?(?:-autostart)?"
-                       "(?:\\.scope|\\.service|\\.slice)$",
-                       0, 0, &error);
+  regex1 = g_regex_new ("^app-(?:[[:alnum:]]+\\-)?(.+?)(?:\\-[[:alnum:]]*)(?:\\.scope|\\.slice)$", 0, 0, &error);
   g_assert (error == NULL);
-  if (g_regex_match (regex, unit, 0, &match))
+  /*
+   * app[-<launcher>]-<ApplicationID>-autostart.service -> no longer true since systemd v248
+   * app[-<launcher>]-<ApplicationID>[@<RANDOM>].service
+   */
+  regex2 = g_regex_new ("^app-(?:[[:alnum:]]+\\-)?(.+?)(?:@[[:alnum:]]*|\\-autostart)?\\.service$", 0, 0, &error);
+  g_assert (error == NULL);
+
+  if (!g_regex_match (regex1, unit, 0, &match))
+    g_clear_pointer (&match, g_match_info_unref);
+
+  if (match == NULL && !g_regex_match (regex2, unit, 0, &match))
+    g_clear_pointer (&match, g_match_info_unref);
+
+  if (match != NULL)
     {
       const char *escaped_app_id;
       /* Unescape the unit name which may have \x hex codes in it, e.g.
@@ -2273,7 +2282,6 @@ xdp_validate_serialized_icon (GVariant  *v,
   const char *allowed_icon_formats[] = { "png", "jpeg", "svg", NULL };
   int size;
   const char *MAX_ICON_SIZE = "512";
-  g_auto(GStrv) parts = NULL;
   gconstpointer bytes_data;
   gsize bytes_len;
   g_autoptr(GKeyFile) key_file = NULL;
@@ -2375,3 +2383,27 @@ xdp_validate_serialized_icon (GVariant  *v,
 
   return TRUE;
 }
+
+gboolean
+xdp_variant_contains_key (GVariant *dictionary,
+                          const char *key)
+{
+  GVariantIter iter;
+
+  g_variant_iter_init (&iter, dictionary);
+  while (TRUE)
+    {
+      g_autoptr(GVariant) entry = NULL;
+      g_autoptr(GVariant) entry_key = NULL;
+
+      entry = g_variant_iter_next_value (&iter);
+      if (!entry)
+        break;
+
+      entry_key = g_variant_get_child_value (entry, 0);
+      if (g_strcmp0 (g_variant_get_string (entry_key, NULL), key) == 0)
+        return TRUE;
+    }
+
+  return FALSE;
+}
