@@ -5,12 +5,12 @@
 #include "glib-backports.h"
 
 #include <libportal/portal.h>
-#include "src/xdp-utils.h"
-#include "src/xdp-impl-dbus.h"
+#include "xdp-utils.h"
+#include "xdp-impl-dbus.h"
 
 #include "utils.h"
 
-extern XdpImplLockdown *lockdown;
+extern XdpDbusImplLockdown *lockdown;
 
 extern char outdir[];
 
