@@ -151,6 +151,7 @@ global_setup (void)
   g_mkdtemp (outdir);
   g_debug ("outdir: %s\n", outdir);
 
+  g_setenv ("XDG_CURRENT_DESKTOP", "test", TRUE);
   g_setenv ("XDG_RUNTIME_DIR", outdir, TRUE);
   g_setenv ("XDG_DATA_HOME", outdir, TRUE);
 
@@ -190,8 +191,9 @@ global_setup (void)
 
   backends_executable = g_test_build_filename (G_TEST_BUILT, "test-backends", NULL);
   argv[0] = backends_executable;
-  argv[1] = g_test_verbose () ? "--verbose" : NULL;
-  argv[2] = NULL;
+  argv[1] = "--backend-name=" BACKEND_BUS_NAME;
+  argv[2] = g_test_verbose () ? "--verbose" : NULL;
+  argv[3] = NULL;
 
   g_debug ("launching test-backend\n");
 
@@ -263,7 +265,7 @@ global_setup (void)
                                           &name_appeared,
                                           NULL);
 
-  portal_dir = g_test_build_filename (G_TEST_DIST, "portals", NULL);
+  portal_dir = g_test_build_filename (G_TEST_BUILT, "portals", "test", NULL);
 
   g_clear_object (&launcher);
   launcher = g_subprocess_launcher_new (G_SUBPROCESS_FLAGS_NONE);
@@ -380,17 +382,6 @@ global_teardown (void)
   g_object_unref (dbus);
 }
 
-#ifdef HAVE_PIPEWIRE
-#define check_pipewire(name)
-#else
-#define check_pipewire(name) \
- if (strcmp (name , "camera") == 0) \
-   { \
-     g_test_skip ("Skipping tests that require pipewire"); \
-     return; \
-   }
-#endif
-
 #ifdef HAVE_GEOCLUE
 #define check_geoclue(name)
 #else
@@ -414,7 +405,6 @@ test_##pp##_exists (void) \
   g_autoptr(GError) error = NULL; \
   g_autofree char *owner = NULL; \
  \
- check_pipewire ( #pp ) \
  check_geoclue ( #pp ) \
  \
   proxy = G_DBUS_PROXY (xdp_dbus_##pp##_proxy_new_sync (session_bus, \
@@ -434,7 +424,7 @@ test_##pp##_exists (void) \
 DEFINE_TEST_EXISTS(account, ACCOUNT, 1)
 DEFINE_TEST_EXISTS(background, BACKGROUND, 2)
 DEFINE_TEST_EXISTS(camera, CAMERA, 1)
-DEFINE_TEST_EXISTS(email, EMAIL, 3)
+DEFINE_TEST_EXISTS(email, EMAIL, 4)
 DEFINE_TEST_EXISTS(file_chooser, FILE_CHOOSER, 3)
 DEFINE_TEST_EXISTS(game_mode, GAME_MODE, 4)
 DEFINE_TEST_EXISTS(inhibit, INHIBIT, 3)
@@ -442,7 +432,7 @@ DEFINE_TEST_EXISTS(location, LOCATION, 1)
 DEFINE_TEST_EXISTS(network_monitor, NETWORK_MONITOR, 3)
 DEFINE_TEST_EXISTS(notification, NOTIFICATION, 1)
 DEFINE_TEST_EXISTS(open_uri, OPEN_URI, 3)
-DEFINE_TEST_EXISTS(print, PRINT, 1)
+DEFINE_TEST_EXISTS(print, PRINT, 2)
 DEFINE_TEST_EXISTS(proxy_resolver, PROXY_RESOLVER, 1)
 DEFINE_TEST_EXISTS(screenshot, SCREENSHOT, 2)
 DEFINE_TEST_EXISTS(settings, SETTINGS, 1)
@@ -493,6 +483,7 @@ main (int argc, char **argv)
   g_test_add_func ("/portal/email/cancel", test_email_cancel);
   g_test_add_func ("/portal/email/close", test_email_close);
   g_test_add_func ("/portal/email/address", test_email_address);
+  g_test_add_func ("/portal/email/punycode_address", test_email_punycode_address);
   g_test_add_func ("/portal/email/subject", test_email_subject);
   g_test_add_func ("/portal/email/parallel", test_email_parallel);
 
